#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import time
import os
from math import fabs
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import *
from .cmapsettings import CMapSettings
from .gui.cmapdlgs import LevelData, MeasParameters, MapDlg, CMapSettingsDlg, check_map_tiles
import csv


APP_TITLE ="Coverage Map View 1.0"


class CMap(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this code unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Base settings
        self._tab_name = "nslcoveragemap"
        self._scr_title = APP_TITLE
        self._scr_description = "Loads a coverage map from a cmap csv-file and displays it."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_icon_close.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        self.settings = {}

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        # Load coverage map settings
        # -------------------------------------------------------------------------------
        self.settings = CMapSettings.load()
        if not self.settings:
            self.MessageBoxModal("Please run App Coverage Map Settings first!",
                                 "Coverage Map", wx.OK | wx.ICON_INFORMATION)
            return

        # Add map display dialog.
        # -------------------------------------------------------------------------------
        # Setup measurement file from Open File Dialog:
        # -------------------------------------------------------------------------------
        dlg = wx.FileDialog(self.frm_dlghelper, "Open CMap CSV file", "", "",
                            "CMap CSV files (*.csv)|*.csv",
                            wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

        if self.ShowDlgModalTop(dlg) != wx.ID_OK:
            dlg.Destroy()
            return
        file_path = dlg.GetPath()
        dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore


        # Get measurement parameters:
        # -------------------------------------------------------------------------------
        self.meas_param = MeasParameters.from_csv_file(file_path)

        # Check unit string:
        # -------------------------------------------------------------------------------
        settings_unit = self.settings.get("unit", "")
        if self.meas_param.unit is None or self.meas_param.unit != settings_unit:
            self.MessageBoxModal("The unit of the measurement file does not match the unit of the settings file!",
                                 "Unit mismatch!", wx.OK | wx.ICON_INFORMATION)
            return

        if not check_map_tiles(self.settings.get("map_path")):
            self.MessageBoxModal("Maps not found. Please re-run the settings to select the "
                                 "\'Mapnik\' folder in the Maps sub-directory.",
                                 "Map Files Error", wx.OK | wx.ICON_INFORMATION)
            self.signalshark.disconnect()
            return

        map_dlg = MapDlg(self.main_gui, APP_TITLE, self.settings, file_path,
                         self.meas_param)
        point_dist = self.settings.get("point_dist", 500)

        #  Run measurement in additional task to keep GUI alive.
        # -------------------------------------------------------------------------------
        mthread = MeasDlgThread(main_gui=self.main_gui, dlg=map_dlg,
                                callback=CMap._run_measurement,
                                args=[file_path])

        # Start the measurement thread and show a visualization (progress) dialog.
        mthread.start_measurement()

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         file_name: str):
        """Main method of the measurement thread that handles a time consuming measurement.

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param master_ip: Custom parameter to handover the file name
        :return:
        """

        data_file = None
        try:
            # Update the dialog message text and show a progress bar
            # -------------------------------------------------------------------------------
            update_progress(msg="Loading data....",
                            show_progress=True, btn_style= wx.OK)

            # Open file:
            data_file = open(file_name, "r")
            data_reader = csv.reader(data_file, delimiter=";")

            # Skip header
            for i in range(18):
                data_file.readline()

            check_cnt = 0
            data_lst = []
            for row in data_reader:
                # Periodically check, whether the user has clicked the cancel button,
                # and exit thread if so.
                if check_cnt >= 300:
                    # Allow the thread to synchronize the stop event.
                    stopevent.wait(0.2)
                    if stopevent.is_set():
                        return
                    update_progress(data=data_lst, force_update = True)
                    data_lst = []
                    check_cnt = 0

                data = LevelData()
                data.timestamp = row[0]
                data.gnss = GnssCoordinate(float(row[1]), float(row[2]))
                data.det_val = float(row[4])
                data_lst.append(data)
                check_cnt += 1
            if len(data_lst) > 0:
                update_progress(data=data_lst, force_update = True)
            return wait_for_stopevent(msg="Loading data finished.", btn_style= wx.OK)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e), icon_style=wx.ICON_ERROR)
        finally:
            # Close file and connection.
            if data_file:
                data_file.close()
